import PFElement from '../../pfelement/dist/pfelement.js';

/*!
 * PatternFly Elements: PfeDocumentation 1.0.0-prerelease.45
 * @license
 * Copyright 2020 Red Hat, Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
*/

// Config for mutation observer to see if things change inside of the component
const lightDomObserverConfig = {
  characterData: true,
  attributes: true,
  subtree: true,
  childList: true
};

class PfeDocumentation extends PFElement {
  static get version() {
    return "1.0.0-prerelease.45";
  }

  get html() {
    return `<style>:host{display:block}:host([hidden]){display:none}
/*# sourceMappingURL=cp-documentation.min.css.map */
</style><div id="wrapper"></div>`;
  }

  static get properties() {
    return {"loaded":{"title":"Loaded","type":"string","enum":[],"default":"","prefixed":true}," endpoint":{"title":" endpoint","type":"string","enum":[],"default":"","prefixed":true}," css":{"title":" css","type":"string","enum":[],"default":"","prefixed":true}};
  }

  static get slots() {
    return {"default":{"title":"Default slot","type":"array","namedSlot":false,"items":{"oneOf":[{"$ref":"raw"}]}}};
  }
  static get tag() {
    return "cp-documentation";
  }

  get schemaUrl() {
    return "cp-documentation.json";
  }

  get templateUrl() {
    return "cp-documentation.html";
  }

  get styleUrl() {
    return "cp-documentation.scss";
  }

  static get events() {
    return {
      change: `${this.tag}:change`,
      loaded: `${this.tag}:loaded`
    };
  }

  // Declare the type of this component
  static get PfeType() {
    return PFElement.PfeTypes.Container;
  }

  static get observedAttributes() {
    return ["pfe-loaded", "pfe-endpoint", "pfe-css"];
  }

  constructor() {
    super(PfeDocumentation, { type: PfeDocumentation.PfeType });

    // Ensure 'this' is tied to the component object in these member functions
    this._processLightDom = this._processLightDom.bind(this);
    this._loadCss = this._loadCss.bind(this);
    this.loadData = this.loadData.bind(this);
    this.getData = this.getData.bind(this);

    // Setup mutation observer to watch for content changes
    this._observer = new MutationObserver(this._processLightDom);

    // Initialize object to store data in
    this._moduleData = {};
  }

  connectedCallback() {
    super.connectedCallback();

    this._processLightDom();

    this._observer.observe(this, lightDomObserverConfig);

    this.addEventListener(PfeDocumentation.events.change, this._changeHandler);
    this.addEventListener(PfeDocumentation.events.loaded, this._loadedHandler);

    if (this.hasAttribute("pfe-css")) {
      this._loadCss();
    }
  }

  disconnectedCallback() {
    this.removeEventListener(
      PfeDocumentation.events.change,
      this._changeHandler
    );
    this.removeEventListener(
      PfeDocumentation.events.loaded,
      this._loadedHandler
    );
  }

  attributeChangedCallback(attr, oldVal, newVal) {
    super.attributeChangedCallback(attr, oldVal, newVal);

    switch (attr) {
      case "pfe-endpoint":
        this.loadData();
        break;
    }
  }

  /**
   * Handle initialization or changes in light DOM
   * Clone them into the shadowRoot
   */
  _processLightDom() {
    // Preventing issues in IE11 & Edge
    if (window.ShadyCSS) {
      this._observer.disconnect();
    }

    const shadowWrapper = this.shadowRoot.getElementById("wrapper"),
      oldContentWrapper = this.shadowRoot.getElementById("content"),
      // An element that light dom content will be put into and replaces the shadowWrapper at the end
      newContentWrapper = document.createElement("div");

    this.childNodes.forEach(childNode => {
      const cloneNode = childNode.cloneNode(true);
      newContentWrapper.append(cloneNode);
    });

    if (oldContentWrapper) {
      shadowWrapper.replaceChild(newContentWrapper, oldContentWrapper);
    } else {
      shadowWrapper.append(newContentWrapper);
    }
    newContentWrapper.setAttribute("id", "content");

    // Reconnecting mutationObserver for IE11 & Edge
    if (window.ShadyCSS) {
      this._observer.observe(this, lightDomObserverConfig);
    }
  }

  _loadCss() {
    // If we have an attribute and the CSS hasn't been loaded yet
    if (
      this.hasAttribute("pfe-css") &&
      !this.shadowRoot.getElementById("pfe-css")
    ) {
      const pfeCssUrl = this.getAttribute("pfe-css");
      const linkTag = document.createElement("link");
      linkTag.setAttribute("id", "pfe-css");
      linkTag.setAttribute("href", pfeCssUrl);
      linkTag.setAttribute("rel", "stylesheet");
      this.shadowRoot.append(linkTag);
    }
  }

  /**
   * Get the content from pfe-endpoint attribute
   *
   * Also loads the body content
   */
  loadData() {
    const endpointUrl = this.getAttribute("pfe-endpoint");
    // @todo local caching?
    if (endpointUrl) {
      fetch(endpointUrl)
        .then(response => response.json())
        .then(data => {
          console.log(data);
          if (typeof data.module === "object") {
            this.setAttribute("pfe-loaded", "");
            this._moduleData = data.module;
            if (this._moduleData.body) {
              this.innerHTML = this._moduleData.body;
            }
          }
        })
        .catch(error => console.log(`${this.tag}: ${error}`));
    }
  }

  /**
   * Allows parent DOM to access the module data
   * @return {object} The module data
   */
  getData() {
    if (Object.keys(this._moduleData).length) {
      return this._moduleData;
    }

    return null;
  }

  _changeHandler(event) {
    this.emitEvent(PfeDocumentation.events.change, {
      detail: {}
    });
  }

  _loadedHandler(event) {
    this.emitEvent(PfeDocumentation.events.loaded, {
      detail: {}
    });
  }
}

PFElement.create(PfeDocumentation);

export default PfeDocumentation;
//# sourceMappingURL=cp-documentation.js.map
